function T = convmtxSV(h,idx1,idx2,M,N,P,Q)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% T = convmtxSV(h,idx,M,N,P,Q) 
% is a function that constructs spatially variant 
% convolution matrices.
% 
% Input: 
% h   - point spread functions
% idx - coordinates of the psfs
%       idx{}(1,:) - coordinate of the row
%       idx{}(2,:) - coordinate of the col
% M,N - size of the image
% P,Q - size of PSF
%
% Output:
% T   - a sparse matrix storing psf values for spatially variant blur
%
% Stanley Chan
% initial version, 2013/1/3
% fixed column-row mismatch, 2016/8/23
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Hcube = zeros(M, P, N, Q);

for t=1:length(idx1)
    for m=1:size(idx1{t},2)
        for n=1:size(idx2{t},2)
            for q=1:Q
                Hcube(idx1{t}(m), :, idx2{t}(n), q) = h{t}(:,q)';
            end
        end
    end
end

T = Hcube2mtx(Hcube);








function T = Hcube2mtx(Hcube)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hcube2mtx coverts Hcube to sparse convolution matrix
%
% Stanley Chan
% UCSD, 2011/3/1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[M P N Q] = size(Hcube);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Shuffles non-zero values
% Stanley Chan, 2009/12/7
%               2011/3/1 fix bug by adding circshift in Hcube_B for loop
%               2011/3/2 fix bug by adding Tmp in Hcube_B for loop
%
% Hcube_B = x x x   ... 
%           o o o   ...
%           ^ ^ ^   ...
%           & & &   ...
%
% Change to
% Hcube_B = x o ^   ...
%           o ^ &   ...
%           ^ & &   ...
%           & & &   ...
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Hcube_A = Hcube;
for i=1:Q
    Hcube_B = circshift(Hcube(:, :, :, i), [floor(P/2), 0, 0]);
    for j=1:P
        Tmp = Hcube_B(1:j-1,j,:); % circular boundary
%         Tmp = repmat(Hcube_B(M,j,:), [j-1,1,1]); % replicate boundary
        Hcube_B(1:M-(j-1) , j , :) = Hcube_B(j:M, j, :);
        Hcube_B(M-(j-2):M , j , :) = Tmp;
    end
    Hcube_A(:, :, :, i) = Hcube_B;
end
Hcube_C = Hcube_A;

for i=1:Q
    Hcube_C(1:M,1:P,1:N-(i-1),i)   = Hcube_A(1:M,1:P,i:N,i);
    Hcube_C(1:M,1:P,N-(i-1)+1:N,i) = Hcube_A(1:M,1:P,1:i-1,i); % circular boundary
%     Hcube_C(1:M,1:P,N-(i-1)+1:N,i) = repmat(Hcube_A(1:M,1:P,N,i), [1 1 i-1 1]); % replicate boundary
end
Hcube_D = circshift(Hcube_C, [0, 0, floor(Q/2), 0]);
Tvals1  = Hcube_D(:);


% Modification of MATLAB convmtx2
% Stanley Chan, 2009/12/6
blockHeight = M + P - 1;
blockWidth = M;
blockNonZeros = P * M;

totalNonZeros = Q * N * blockNonZeros;

THeight = (N+Q-1)*blockHeight;
TWidth = N*blockWidth;

Trows = zeros(totalNonZeros,1);
Tcols = zeros(totalNonZeros,1);

c = repmat((1:M)',P,1);
r = compute_r(M, P);

r = r(:, ones(1, N));
c = c(:, ones(1, N));

colOffsets = ((1:N) - 1)*M;
colOffsets = colOffsets(ones(M*P,1), :);
colOffsets = colOffsets + c;
colOffsets = colOffsets(:);

rowOffsets = ((1:N) - 1) * blockHeight;
rowOffsets = rowOffsets(ones(M*P,1), :);
rowOffsets = rowOffsets + r;
rowOffsets = rowOffsets(:);
  
for k = 1:Q
  first = (k-1)*N*blockNonZeros + 1;
  last = first + N*blockNonZeros - 1;
  Trows(first:last) = rowOffsets;
  Tcols(first:last) = colOffsets;
  rowOffsets = rowOffsets + blockHeight;
end

T = sparse(Trows, Tcols, Tvals1, THeight, TWidth);
PP = false(M+P-1, N+Q-1);
PP(1:M, :) = true;
idx = PP==false;
T(idx(:), :) = [];

[MT NT] = size(T);

% Modified outermost boundary condition
% Stanley Chan 2009/12/29
if mod(Q,2)==0
    Tb= T(MT-M*(floor(Q/2)-1)+1:MT, :);
    T(MT-M*(floor(Q/2))+1:MT, :) = [];
    Tp= T(1:M*(floor(Q/2)), :);
    T(1:M*(floor(Q/2)-1), :) = [];
    T(M*N-M*(floor(Q/2))+1:M*N, :) = T(M*N-M*(floor(Q/2))+1:M*N, :) + Tp;
    T(1:M*(floor(Q/2)-1), :) = T(1:M*(floor(Q/2)-1), :) + Tb;
else
    Tb = T(MT-M*(floor(Q/2))+1:MT, :); 
    T(MT-M*(floor(Q/2))+1:MT, :) = [];
    Tp = T(1:M*(floor(Q/2)), :);
    T(1:M*(floor(Q/2)), :)       = [];
    T(M*N-M*(floor(Q/2))+1:M*N, :) = T(M*N-M*(floor(Q/2))+1:M*N, :) + Tp;
    T(1:M*(floor(Q/2)), :) = T(1:M*(floor(Q/2)), :) + Tb;
end


% % Normalization
% % Stanley Chan 2011/2/28
% SumT = sum(T,2);
% [M N] = size(T);
% T = spdiags(1./max(SumT,1e-5),0,M,N)*T;









function t = compute_r(M, P)
% Compute the index
% Stanley Chan 2009/12/6


P2 = floor(P/2);

r = [P2:-1:1 M:-1:1 M:-1:M-P2+1];
c = r;

r = r(:);                               % force column structure
p = length(r);
x = [r(p:-1:2) ; c(:)];                 % build vector of user data
xclass = class(x);
cidx = (zeros(xclass):M-1)';
ridx = ones(xclass):P;
t = cidx(:,ones(P,1)) + ridx(ones(M,1),:);  % Toeplitz subscripts
t = x(t);
t = t(:);
